@@ -35,8 +35,21 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+function browserWindowsCount() {
+  let count = 0;
+  let e = Services.wm.getEnumerator("navigator:browser");
+  while (e.hasMoreElements()) {
+    if (!e.getNext().closed)
+      ++count;
+  }
+  return count;
+}
+
 function test() {
-  /** Test for Bug 394759 **/
+  // This test takes quite some time, and timeouts frequently, so we require
+  // more time to run.
+  // See Bug 518970.
+  requestLongerTimeout(2);
   
   // test setup
   let ss = Cc["@mozilla.org/browser/sessionstore;1"].getService(Ci.nsISessionStore);
@@ -56,56 +69,63 @@ function test() {
     gPrefService.setIntPref("browser.sessionstore.max_windows_undo", max_windows_undo + 1);
     let closedWindowCount = ss.getClosedWindowCount();
   
-    let newWin = openDialog(location, "_blank", "chrome,all,dialog=no", testURL);
+    let newWin = openDialog(location, "", "chrome,all,dialog=no", testURL);
     newWin.addEventListener("load", function(aEvent) {
-      newWin.gBrowser.addEventListener("load", function(aEvent) {
-        newWin.gBrowser.removeEventListener("load", arguments.callee, true);
+      newWin.gBrowser.selectedBrowser.addEventListener("load", function(aEvent) {
+        newWin.gBrowser.selectedBrowser.removeEventListener("load", arguments.callee, true);
 
         executeSoon(function() {
-          newWin.gBrowser.addTab();
+          newWin.gBrowser.addTab().linkedBrowser.stop();
+          executeSoon(function() {
+            // mark the window with some unique data to be restored later on
+            ss.setWindowValue(newWin, uniqueKey, uniqueValue);
+            let textbox = newWin.content.document.getElementById("textbox");
+            textbox.value = uniqueText;
 
-          // mark the window with some unique data to be restored later on
-          ss.setWindowValue(newWin, uniqueKey, uniqueValue);
-          let textbox = newWin.content.document.getElementById("textbox");
-          textbox.wrappedJSObject.value = uniqueText;
+            newWin.close();
 
-          newWin.close();
+            is(ss.getClosedWindowCount(), closedWindowCount + 1,
+               "The closed window was added to Recently Closed Windows");
+            let data = JSON.parse(ss.getClosedWindowData())[0];
+            ok(data.title == testURL && JSON.stringify(data).indexOf(uniqueText) > -1,
+               "The closed window data was stored correctly");
 
-          is(ss.getClosedWindowCount(), closedWindowCount + 1,
-             "The closed window was added to Recently Closed Windows");
-          let data = JSON.parse(ss.getClosedWindowData())[0];
-          ok(data.title == testURL && data.toSource().indexOf(uniqueText) > -1,
-             "The closed window data was stored correctly");
+            // reopen the closed window and ensure its integrity
+            let newWin2 = ss.undoCloseWindow(0);
 
-          // reopen the closed window and ensure its integrity
-          let newWin2 = ss.undoCloseWindow(0);
+            ok(newWin2 instanceof ChromeWindow,
+               "undoCloseWindow actually returned a window");
+            is(ss.getClosedWindowCount(), closedWindowCount,
+               "The reopened window was removed from Recently Closed Windows");
 
-          ok(newWin2 instanceof ChromeWindow,
-             "undoCloseWindow actually returned a window");
-          is(ss.getClosedWindowCount(), closedWindowCount,
-             "The reopened window was removed from Recently Closed Windows");
+            // SSTabRestored will fire more than once, so we need to make sure we count them
+            let restoredTabs = 0;
+            let expectedTabs = data.tabs.length;
+            newWin2.addEventListener("load", function(aEvent) {
+              newWin2.gBrowser.tabContainer.addEventListener("SSTabRestored", function(aEvent) {
+                if (++restoredTabs < expectedTabs)
+                  return;
+                newWin2.gBrowser.tabContainer.removeEventListener("SSTabRestored", arguments.callee, true);
 
-          newWin2.addEventListener("load", function(aEvent) {
-            newWin2.gBrowser.addEventListener("SSTabRestored", function(aEvent) {
-              newWin2.gBrowser.removeEventListener("SSTabRestored", arguments.callee, true);
+                is(newWin2.gBrowser.tabs.length, 2,
+                   "The window correctly restored 2 tabs");
+                is(newWin2.gBrowser.currentURI.spec, testURL,
+                   "The window correctly restored the URL");
 
-              is(newWin2.gBrowser.tabContainer.childNodes.length, 2,
-                 "The window correctly restored 2 tabs");
-              is(newWin2.gBrowser.selectedBrowser.currentURI.spec, testURL,
-                 "The window correctly restored the URL");
+                let textbox = newWin2.content.document.getElementById("textbox");
+                is(textbox.value, uniqueText,
+                   "The window correctly restored the form");
+                is(ss.getWindowValue(newWin2, uniqueKey), uniqueValue,
+                   "The window correctly restored the data associated with it");
 
-              let textbox = newWin2.content.document.getElementById("textbox");
-              is(textbox.wrappedJSObject.value, uniqueText,
-                 "The window correctly restored the form");
-              is(ss.getWindowValue(newWin2, uniqueKey), uniqueValue,
-                 "The window correctly restored the data associated with it");
-
-              // clean up
-              newWin2.close();
-              gPrefService.clearUserPref("browser.sessionstore.max_windows_undo");
-              executeSoon(callback);
-            }, true);
-          }, false);
+                // clean up
+                newWin2.close();
+                if (gPrefService.prefHasUserValue("browser.sessionstore.max_windows_undo"))
+                  gPrefService.clearUserPref("browser.sessionstore.max_windows_undo");
+                executeSoon(callback);
+              }, true);
+            }, false);
+          });
         });
       }, true);
     }, false);
@@ -137,17 +157,18 @@ function test() {
       let winData = windowsToOpen.shift();
       let settings = "chrome,dialog=no," +
                      (winData.isPopup ? "all=no" : "all");
-      let url = "http://window" + windowsToOpen.length + ".example.com";
-      let window = openDialog(location, "_blank", settings, url);
-      window.addEventListener("load", function(aEvent) {
-        window.gBrowser.addEventListener("load", function(aEvent) {
+      let url = "http://example.com/?window=" + windowsToOpen.length;
+      let win = openDialog(location, "", settings, url);
+      win.addEventListener("load", function(aEvent) {
+        win.gBrowser.selectedBrowser.addEventListener("DOMContentLoaded", function(aEvent) {
+          win.gBrowser.selectedBrowser.removeEventListener("DOMContentLoaded", arguments.callee, true);
           // the window _should_ have state with a tab of url, but it doesn't
           // always happend before window.close(). addTab ensure we don't treat
           // this window as a stateless window
-          window.gBrowser.addTab();
-          window.gBrowser.removeEventListener("load", arguments.callee, true);
+          win.gBrowser.addTab();
+
           executeSoon(function() {
-            window.close();
+            win.close();
             executeSoon(function() {
               openWindowRec(windowsToOpen, expectedResults, recCallback);
             });
@@ -185,6 +206,11 @@ function test() {
 
     // backup old state
     let oldState = ss.getBrowserState();
+    let oldState_wins = JSON.parse(oldState).windows.length;
+    if (oldState_wins != 1) {
+      ok(false, "oldState in test_purge has " + oldState_wins + " windows instead of 1");
+    }
+
     // create a new state for testing
     const REMEMBER = Date.now(), FORGET = Math.random();
     let testState = {
@@ -251,30 +277,30 @@ function test() {
     let closedWindowData = JSON.parse(ss.getClosedWindowData());
 
     // First set of tests for _closedWindows[0] - tests basics
-    let window = closedWindowData[0];
-    is(window.tabs.length, 1, "1 tab was removed");
-    is(countOpenTabsByTitle(window.tabs, FORGET), 0,
+    let win = closedWindowData[0];
+    is(win.tabs.length, 1, "1 tab was removed");
+    is(countOpenTabsByTitle(win.tabs, FORGET), 0,
        "The correct tab was removed");
-    is(countOpenTabsByTitle(window.tabs, REMEMBER), 1,
+    is(countOpenTabsByTitle(win.tabs, REMEMBER), 1,
        "The correct tab was remembered");
-    is(window.selected, 1, "Selected tab has changed");
-    is(window.title, REMEMBER, "The window title was correctly updated");
+    is(win.selected, 1, "Selected tab has changed");
+    is(win.title, REMEMBER, "The window title was correctly updated");
 
     // Test more complicated case 
-    window = closedWindowData[1];
-    is(window.tabs.length, 3, "2 tabs were removed");
-    is(countOpenTabsByTitle(window.tabs, FORGET), 0,
+    win = closedWindowData[1];
+    is(win.tabs.length, 3, "2 tabs were removed");
+    is(countOpenTabsByTitle(win.tabs, FORGET), 0,
        "The correct tabs were removed");
-    is(countOpenTabsByTitle(window.tabs, REMEMBER), 3,
+    is(countOpenTabsByTitle(win.tabs, REMEMBER), 3,
        "The correct tabs were remembered");
-    is(window.selected, 3, "Selected tab has changed");
-    is(window.title, REMEMBER, "The window title was correctly updated");
+    is(win.selected, 3, "Selected tab has changed");
+    is(win.title, REMEMBER, "The window title was correctly updated");
 
     // Tests handling of _closedTabs
-    window = closedWindowData[2];
-    is(countClosedTabsByTitle(window._closedTabs, REMEMBER), 1,
+    win = closedWindowData[2];
+    is(countClosedTabsByTitle(win._closedTabs, REMEMBER), 1,
        "The correct number of tabs were removed, and the correct ones");
-    is(countClosedTabsByTitle(window._closedTabs, FORGET), 0,
+    is(countClosedTabsByTitle(win._closedTabs, FORGET), 0,
        "All tabs to be forgotten were indeed removed");
 
     // restore pre-test state
@@ -282,9 +308,13 @@ function test() {
     executeSoon(callback);
   }
   
+  is(browserWindowsCount(), 1, "Only one browser window should be open initially");
   test_basic(function() {
+    is(browserWindowsCount(), 1, "number of browser windows after test_basic");
     test_behavior(function() {
+      is(browserWindowsCount(), 1, "number of browser windows after test_behavior");
       test_purge(function() {
+        is(browserWindowsCount(), 1, "number of browser windows after test_purge");
         finish();
       });
     });
