@@ -344,7 +344,6 @@ public:
   nsresult FlushTags();
 
   PRBool   IsCurrentContainer(nsHTMLTag mType);
-  PRBool   IsAncestorContainer(nsHTMLTag mType);
 
   void DidAddContent(nsIContent* aContent);
   void UpdateChildCounts();
@@ -664,21 +663,6 @@ SinkContext::IsCurrentContainer(nsHTMLTag aTag)
   return PR_FALSE;
 }
 
-PRBool
-SinkContext::IsAncestorContainer(nsHTMLTag aTag)
-{
-  PRInt32 stackPos = mStackPos - 1;
-
-  while (stackPos >= 0) {
-    if (aTag == mStack[stackPos].mType) {
-      return PR_TRUE;
-    }
-    stackPos--;
-  }
-
-  return PR_FALSE;
-}
-
 void
 SinkContext::DidAddContent(nsIContent* aContent)
 {
@@ -2465,9 +2449,8 @@ HTMLContentSink::AddDocTypeDecl(const nsIParserNode& aNode)
     nsAutoString voidString;
     voidString.SetIsVoid(PR_TRUE);
     rv = NS_NewDOMDocumentType(getter_AddRefs(docType),
-                               mDocument->NodeInfoManager(), nsnull,
-                               nameAtom, nsnull, nsnull, publicId, systemId,
-                               voidString);
+                               mDocument->NodeInfoManager(), nsnull, nameAtom,
+                               publicId, systemId, voidString);
     NS_ENSURE_SUCCESS(rv, rv);
 
     if (oldDocType) {
